// File: MaxWithAssertions.java
// A Java application that demonstrates the use of assertions.
// See pages 30-34 of our textbook.

/******************************************************************************
* The MaxWithAssertions Java application demonstrates the use of assertions.
*
******************************************************************************/
public class MaxWithAssertions
{
   /**
   * The main() method tests the maxOf3() method and the maxOfArray() method..
   **/
   public static void main(String[ ] args)
   {
      System.out.println( maxOf3(3, 7, 5) );

      int[] a = {4, 5, 6, 9, 8, 7};

      System.out.println( maxOfArray(a) );
   }


   /**
   * Returns the largest of three int values.
   *
   * @param a b c
   *   any three int numbers
   * @precondition
   *   none.
   * @return
   *   the maximum value from the three arguments a, b, and c.
   * @throws java.lang.AssertionError
   *   Indicates that something is wrong.
   **/
   public static int maxOf3(int a, int b, int c)
   {
      // Set answer to the largest of a, b, and c.
      int answer = a;  // Initially set answer to a.
      if (b > answer)  // Maybe change the answer to b.
         answer = b;
      if (c > answer)  // Maybe change the answer to c.
         answer = c;

      // Check that the computation did what we expected.
      assert (answer == a) || (answer == b) || (answer == c)
         : "maxOf3 answer is not equal to one of the arguments";
      assert (answer >= a) & (answer >= b) && (answer >= c)
         : "maxOf3 answer is not equal to the largest argument";

      return answer;
   }


   /**
   * Returns the largest value in an int array.
   *
   * @param a
   *   a non-empty array of ints (the length must not be zero)
   * @precondition
   *   a.length > 0
   * @return
   *   the maximum value from the array a.
   * @throws java.lang.ArrayIndexOutOfBounds
   *   Indicates that the array length is zero.
   * @throws java.lang.AssertionError
   *   Indicates that something is wrong.
   **/
   public static int maxOfArray(int[] a)
   {
      // Set answer to the largest value in the arrray.
      int answer = a[0];  // Initially set answer to the first element.
      for (int i = 1; i < a.length; i++)
      {
         if (a[i] > answer)  // Maybe change the answer to a[i]
            answer = a[i];
      }

      // Check that the computation did what we expected.
      assert contains(a, answer)
         : "maxOfArray answer is not in the array";
      assert greaterOrEqual(a, answer)
         : "maxOfArray answer is less than an array element";

      return answer;
   }


   // This private method checks to make sure that the specified value is contained somewhere
   // in the array a. The return value is true if the value is found; otherwise, the return value
   // is false.
   private static boolean contains(int[] a, int value)
   {
      for (int i = 0; i < a.length; i++)
      {
         if (a[i] == value)
            return true;
      }
      // The loop ended without finding the specified value.
      return false;
   }


   // This private method checks to make sure that the specified value is greater than or equal to
   // every element in the array a. In this case, the method returns true. On the other hand, if the
   // specified value is less than some element in the array, then the method returns false.
   private static boolean greaterOrEqual(int[] a, int value)
   {
      for (int i = 0; i < a.length; i++)
      {
         if (a[i] > value)
            return false;
      }
      // The loop finished without finding an aray elemt that exceeds the value,
      // so we return true.
      return true;
   }
}